/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Asynchronous Bus Driver                                          */
/*  EventID:   Insert Extended Event-ID Test Point                              */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   32a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for FROM Control                                             */
/*==============================================================================*/
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

#include "dt_tpdrv.h"

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DEVNAME                                                             */
/*  Desc:   Register Device file name for Linux.                                */
/*==============================================================================*/
#define DEVNAME "/dev/dtdrvasync"

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02	/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08	/* EventTrigger */

static int fd = -1;

/*==============================================================================*/
/*  Func:   _TP_BusOutOpen                                                      */
/*  Desc:   Open device driver for Test Point.                                  */
/*==============================================================================*/
void _TP_BusOutOpen(void)
{
	fd = open(DEVNAME, O_RDWR);
	if( fd < 0 )
	{
		perror("open");
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOutClose                                                     */
/*  Desc:   Close device driver for Test Point.                                 */
/*==============================================================================*/
void _TP_BusOutClose(void)
{
	if( fd >= 0 )
	{
		close(fd);
		fd = -1;
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	int	ret;
	dt_tp	tp;	

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr;
	tp.data[0]	= dat;
	tp.count	= 1;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable Test Point                                       */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	dt_tp	tp;	
	int	i, cnt = 0;
	int	ret;
	unsigned char *p;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr | DT_VARIABLE_BIT;			/* Memory Dump Start */ 
	tp.data[0]	= dat;
	cnt += 1;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			tp.data[cnt] = p[i];
			if ((i + 1) < size) {
				tp.data[cnt] |= p[i+1] << 8;
			}
			tp.addr[cnt] = DT_VARIABLE_BIT;
			if (i == 0) {
				tp.addr[cnt] |= size << 8;
			}
			cnt++;
		}
	}
	tp.count = cnt;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}

/*==============================================================================*/
/*  Func:   _TP_EventTrigger32                                                  */
/*  Desc:   Called by Event Trigger 32bit Test Point                            */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat,DT_UINT event_id)
{
	dt_tp	tp;	
	int	ret;

	if (fd < 0) {
		_TP_BusOutOpen();
	}

	tp.addr[0]	= addr | DT_EVTTRG_BIT ;
	tp.data[0]	= ((unsigned short)dat);
	tp.addr[1]	= DT_EVTTRG_BIT ;
	tp.data[1]	= ((unsigned short)((event_id) & 0x0000FFFF));
	tp.addr[2]	= DT_EVTTRG_BIT ;
	tp.data[2]	= ((unsigned short)((event_id >> 8) & 0x0000FFFF));
	tp.count	= 3;
	ret = write( fd, &tp, sizeof(tp) );
	if( ret < 0 )
	{
		perror("write");
	}
}
/*==============================================================================*/
