/*==============================================================================*/
/*  Copyright (C) 2020, Heartland Data inc. All Rights Reserved.                */
/*                                                                              */
/*  Title  :   Asynchronous Bus Driver                                          */
/*  EventID:   Insert Extended Event-ID Test Point                              */
/*  AppVer :   DT+ Ver1.00`                                                    */
/*  FileID :   32a                                                              */
/*  Version:   1.0                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for FROM Control                                             */
/*==============================================================================*/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/miscdevice.h>

#include "dt_tpdrv.h"

#define LICENSE "GPL v2"

static void *cookie = NULL;

/*==============================================================================*/
/*  Desc:   Please set beginning address and size in the virtual memory space.  */
/*==============================================================================*/
#define	FLASH_ADDR	0xF0000000
#define	FLASH_SIZE	0x02000000

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT+ Project setting.     */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02			/* Memory Dump MASK */
#define		DT_EVTTRG_BIT			0x08			/* EventTrigger */

/*==============================================================================*/
/*  Func:   async_write                                                         */
/*  Desc:   Test Point Output Function                                          */
/*==============================================================================*/
static void async_write(dt_tp tp)
{
	unsigned char *p;
	int	i;

	if (cookie == NULL) {
		cookie = ioremap(FLASH_ADDR, FLASH_SIZE);
	}

	for( i = 0; i < tp.count; i++ )
	{
		p = (unsigned char *)cookie;
		p += tp.addr[i];
		outw( tp.data[i], (unsigned int)p );
		/* *(unsigned short *)p = (unsigned short)tp.data[i]; */
	}
}

/*==============================================================================*/
/*  Func:   _TP_BusOut                                                          */
/*  Desc:   Called by Test Point                                                */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{
	dt_tp	tp;	

	tp.addr[0]	= addr;
	tp.data[0]	= dat;
	tp.count	= 1;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_BusOut);

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable TestPoint                                        */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size)
{
	dt_tp	tp;	
	int	i, cnt = 0;
	unsigned char *p;

	tp.addr[0]	= addr | DT_VARIABLE_BIT;		/* Memory Dump Start */ 
	tp.data[0]	= dat;
	cnt += 1;

	if (size >= MAX_TP_DATA) size = MAX_TP_DATA;
	if (value) {
		p = (unsigned char *)value;
		for (i = 0; i < size; i += 2) {
			tp.data[cnt] = p[i];
			if ((i + 1) < size) {
				tp.data[cnt] |= p[i+1] << 8;
			}
			tp.addr[cnt] = DT_VARIABLE_BIT;
			if (i == 0) {
				tp.addr[cnt] |= size << 8;
			}
			cnt++;
		}
	}
	tp.count = cnt;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_MemoryOutput);

/*==============================================================================*/
/*  Func:   _TP_EventTrigger                                                    */
/*  Desc:   Called by Event Trigger TestPoint                                   */
/*==============================================================================*/
void _TP_EventTrigger32( DT_UINT addr, DT_UINT dat,DT_UINT event_id)
{
	dt_tp	tp;

	tp.addr[0]	= addr | DT_EVTTRG_BIT ;
	tp.data[0]	= ((unsigned short)dat);
	tp.addr[1]	= DT_EVTTRG_BIT ;
	tp.data[1]	= ((unsigned short)((event_id) & 0x0000FFFF));
	tp.addr[2]	= DT_EVTTRG_BIT ;
	tp.data[2]	= ((unsigned short)((event_id >> 16) & 0x0000FFFF));
	tp.count	= 3;
	async_write(tp);
}
EXPORT_SYMBOL(_TP_EventTrigger32);

/********************************************************************************/

/*==============================================================================*/
/*  Func:   dtdrvasync_open                                                     */
/*  Desc:   This function is executed when opening the device driver.           */
/*==============================================================================*/
static int dtdrvasync_open(struct inode *inode, struct file *filp)
{
	if (cookie == NULL) {
		cookie = ioremap(FLASH_ADDR, FLASH_SIZE);
	}
	printk("dtdrvasync: open\n");
	return 0;
}

/*==============================================================================*/
/*  Func:   dtdrvasync_write                                                    */
/*  Desc:   This function is executed upon write instruction.                   */
/*==============================================================================*/
static int dtdrvasync_write(struct file *filp, char *buff, size_t count, loff_t *pos)
{

	dt_tp *tp;
	unsigned char *p;
	int	i;

	tp = (dt_tp *)buff ;
	for( i = 0; i < tp->count; i++ )
	{
		p = (unsigned char *)cookie ;
		p += tp->addr[i];
		outw( tp->data[i], (unsigned int)p );
		/* *(unsigned short *)p = (unsigned short)tp->data[i]; */
	}
	return count ;

}

/*==============================================================================*/
/*  Func:   dtdrvasync_release                                                  */
/*  Desc:   This function is executed when closing the device driver.           */
/*==============================================================================*/
static int dtdrvasync_release(struct inode *inode, struct file *filp)
{
	iounmap( cookie );
	printk("dtdrvasync: release\n");
	return 0;
}

/*==============================================================================*/
/*  Desc:   file_operations driver_fops                                         */
/*==============================================================================*/
static struct file_operations driver_fops = {
	.write = dtdrvasync_write,
	.open = dtdrvasync_open,
	.release = dtdrvasync_release,
};

/*==============================================================================*/
/*  Desc:   miscdevice misc                                                     */
/*==============================================================================*/
static struct miscdevice misc = {
	.minor = MISC_DYNAMIC_MINOR,
	.name = "dtdrvasync",
	.fops = &driver_fops,
};

/*==============================================================================*/
/*  Func:   init_module_async                                                   */
/*  Desc:   This function is executed when installing the device driver.        */
/*==============================================================================*/
int __init init_module_async(void)
{
	int ret;
	
	ret = misc_register(&misc);
	if (ret < 0) {
		printk(KERN_ERR "dtdrvasync: misc_register() failed\n");
	}
	printk("dtdrvasync: Success init_module.\n");

	return 0;
}

/*==============================================================================*/
/*  Func:   cleanup_module_async                                                */
/*  Desc:   This function is executed when uninstalling the device driver.      */
/*==============================================================================*/
void __exit cleanup_module_async(void)
{
	misc_deregister(&misc);
	printk("dtdrvasync: cleanup_module\n");
}

module_init(init_module_async);
module_exit(cleanup_module_async);

MODULE_LICENSE(LICENSE);
